"""mcp tools for a2a agent interaction."""

from synqed_mcp.tools.salesforce import query_leads
from synqed_mcp.tools.zoom import create_meeting
from synqed_mcp.tools.content_creator import generate_content

__all__ = ["query_leads", "create_meeting", "generate_content"]

